"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SecretStorage = exports.SECRET_STORAGE_ALGORITHM_V1_CURVE25519 = exports.SECRET_STORAGE_ALGORITHM_V1_AES = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _events = require("events");

var _logger = require("../logger");

var olmlib = _interopRequireWildcard(require("./olmlib"));

var _randomstring = require("../randomstring");

var _aes = require("./aes");

/*
Copyright 2019, 2020 The Matrix.org Foundation C.I.C.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var SECRET_STORAGE_ALGORITHM_V1_AES = "m.secret_storage.v1.aes-hmac-sha2"; // don't use curve25519 for writing data.

exports.SECRET_STORAGE_ALGORITHM_V1_AES = SECRET_STORAGE_ALGORITHM_V1_AES;
var SECRET_STORAGE_ALGORITHM_V1_CURVE25519 = "m.secret_storage.v1.curve25519-aes-sha2";
exports.SECRET_STORAGE_ALGORITHM_V1_CURVE25519 = SECRET_STORAGE_ALGORITHM_V1_CURVE25519;
var ZERO_STR = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
/**
 * Implements Secure Secret Storage and Sharing (MSC1946)
 * @module crypto/SecretStorage
 */

var SecretStorage =
/*#__PURE__*/
function (_EventEmitter) {
  (0, _inherits2["default"])(SecretStorage, _EventEmitter);

  function SecretStorage(baseApis, cryptoCallbacks, crossSigningInfo) {
    var _this;

    (0, _classCallCheck2["default"])(this, SecretStorage);
    _this = (0, _possibleConstructorReturn2["default"])(this, (0, _getPrototypeOf2["default"])(SecretStorage).call(this));
    _this._baseApis = baseApis;
    _this._cryptoCallbacks = cryptoCallbacks;
    _this._crossSigningInfo = crossSigningInfo;
    _this._requests = {};
    _this._incomingRequests = {};
    return _this;
  }

  (0, _createClass2["default"])(SecretStorage, [{
    key: "getDefaultKeyId",
    value: function () {
      var _getDefaultKeyId = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee() {
        var defaultKey;
        return _regenerator["default"].wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return this._baseApis.getAccountDataFromServer('m.secret_storage.default_key');

              case 2:
                defaultKey = _context.sent;

                if (defaultKey) {
                  _context.next = 5;
                  break;
                }

                return _context.abrupt("return", null);

              case 5:
                return _context.abrupt("return", defaultKey.key);

              case 6:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function getDefaultKeyId() {
        return _getDefaultKeyId.apply(this, arguments);
      }

      return getDefaultKeyId;
    }()
  }, {
    key: "setDefaultKeyId",
    value: function setDefaultKeyId(keyId) {
      var _this2 = this;

      return new Promise(
      /*#__PURE__*/
      function () {
        var _ref = (0, _asyncToGenerator2["default"])(
        /*#__PURE__*/
        _regenerator["default"].mark(function _callee2(resolve, reject) {
          var listener;
          return _regenerator["default"].wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  listener = function listener(ev) {
                    if (ev.getType() === 'm.secret_storage.default_key' && ev.getContent().key === keyId) {
                      _this2._baseApis.removeListener('accountData', listener);

                      resolve();
                    }
                  };

                  _this2._baseApis.on('accountData', listener);

                  _context2.prev = 2;
                  _context2.next = 5;
                  return _this2._baseApis.setAccountData('m.secret_storage.default_key', {
                    key: keyId
                  });

                case 5:
                  _context2.next = 11;
                  break;

                case 7:
                  _context2.prev = 7;
                  _context2.t0 = _context2["catch"](2);

                  _this2._baseApis.removeListener('accountData', listener);

                  reject(_context2.t0);

                case 11:
                case "end":
                  return _context2.stop();
              }
            }
          }, _callee2, null, [[2, 7]]);
        }));

        return function (_x, _x2) {
          return _ref.apply(this, arguments);
        };
      }());
    }
    /**
     * Add a key for encrypting secrets.
     *
     * @param {string} algorithm the algorithm used by the key.
     * @param {object} opts the options for the algorithm.  The properties used
     *     depend on the algorithm given.
     * @param {string} [keyId] the ID of the key.  If not given, a random
     *     ID will be generated.
     *
     * @return {string} the ID of the key
     */

  }, {
    key: "addKey",
    value: function () {
      var _addKey = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee3(algorithm, opts, keyId) {
        var keyData, _ref2, iv, mac;

        return _regenerator["default"].wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                keyData = {
                  algorithm: algorithm
                };
                if (!opts) opts = {};

                if (opts.name) {
                  keyData.name = opts.name;
                }

                _context3.t0 = algorithm;
                _context3.next = _context3.t0 === SECRET_STORAGE_ALGORITHM_V1_AES ? 6 : 16;
                break;

              case 6:
                if (opts.passphrase) {
                  keyData.passphrase = opts.passphrase;
                }

                if (!opts.key) {
                  _context3.next = 15;
                  break;
                }

                _context3.next = 10;
                return SecretStorage._calculateKeyCheck(opts.key);

              case 10:
                _ref2 = _context3.sent;
                iv = _ref2.iv;
                mac = _ref2.mac;
                keyData.iv = iv;
                keyData.mac = mac;

              case 15:
                return _context3.abrupt("break", 17);

              case 16:
                throw new Error("Unknown key algorithm ".concat(opts.algorithm));

              case 17:
                if (keyId) {
                  _context3.next = 22;
                  break;
                }

              case 18:
                keyId = (0, _randomstring.randomString)(32);

              case 19:
                _context3.next = 21;
                return this._baseApis.getAccountDataFromServer("m.secret_storage.key.".concat(keyId));

              case 21:
                if (_context3.sent) {
                  _context3.next = 18;
                  break;
                }

              case 22:
                _context3.next = 24;
                return this._crossSigningInfo.signObject(keyData, 'master');

              case 24:
                _context3.next = 26;
                return this._baseApis.setAccountData("m.secret_storage.key.".concat(keyId), keyData);

              case 26:
                return _context3.abrupt("return", keyId);

              case 27:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function addKey(_x3, _x4, _x5) {
        return _addKey.apply(this, arguments);
      }

      return addKey;
    }()
    /**
     * Signs a given secret storage key with the cross-signing master key.
     *
     * @param {string} [keyId = default key's ID] The ID of the key to sign.
     *     Defaults to the default key ID if not provided.
     */

  }, {
    key: "signKey",
    value: function () {
      var _signKey = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee4(keyId) {
        var keyInfo;
        return _regenerator["default"].wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                if (keyId) {
                  _context4.next = 4;
                  break;
                }

                _context4.next = 3;
                return this.getDefaultKeyId();

              case 3:
                keyId = _context4.sent;

              case 4:
                if (keyId) {
                  _context4.next = 6;
                  break;
                }

                throw new Error("signKey requires a key ID");

              case 6:
                _context4.next = 8;
                return this._baseApis.getAccountDataFromServer("m.secret_storage.key.".concat(keyId));

              case 8:
                keyInfo = _context4.sent;

                if (keyInfo) {
                  _context4.next = 11;
                  break;
                }

                throw new Error("Key ".concat(keyId, " does not exist in account data"));

              case 11:
                _context4.next = 13;
                return this._crossSigningInfo.signObject(keyInfo, 'master');

              case 13:
                _context4.next = 15;
                return this._baseApis.setAccountData("m.secret_storage.key.".concat(keyId), keyInfo);

              case 15:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function signKey(_x6) {
        return _signKey.apply(this, arguments);
      }

      return signKey;
    }()
    /**
     * Get the key information for a given ID.
     *
     * @param {string} [keyId = default key's ID] The ID of the key to check
     *     for. Defaults to the default key ID if not provided.
     * @returns {Array?} If the key was found, the return value is an array of
     *     the form [keyId, keyInfo].  Otherwise, null is returned.
     */

  }, {
    key: "getKey",
    value: function () {
      var _getKey = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee5(keyId) {
        var keyInfo;
        return _regenerator["default"].wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                if (keyId) {
                  _context5.next = 4;
                  break;
                }

                _context5.next = 3;
                return this.getDefaultKeyId();

              case 3:
                keyId = _context5.sent;

              case 4:
                if (keyId) {
                  _context5.next = 6;
                  break;
                }

                return _context5.abrupt("return", null);

              case 6:
                _context5.next = 8;
                return this._baseApis.getAccountDataFromServer("m.secret_storage.key." + keyId);

              case 8:
                keyInfo = _context5.sent;
                return _context5.abrupt("return", keyInfo ? [keyId, keyInfo] : null);

              case 10:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function getKey(_x7) {
        return _getKey.apply(this, arguments);
      }

      return getKey;
    }()
    /**
     * Check whether we have a key with a given ID.
     *
     * @param {string} [keyId = default key's ID] The ID of the key to check
     *     for. Defaults to the default key ID if not provided.
     * @return {boolean} Whether we have the key.
     */

  }, {
    key: "hasKey",
    value: function () {
      var _hasKey = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee6(keyId) {
        return _regenerator["default"].wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _context6.next = 2;
                return this.getKey(keyId);

              case 2:
                return _context6.abrupt("return", !!_context6.sent);

              case 3:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function hasKey(_x8) {
        return _hasKey.apply(this, arguments);
      }

      return hasKey;
    }()
  }, {
    key: "keyNeedsUpgrade",
    value: function () {
      var _keyNeedsUpgrade = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee7(keyId) {
        var keyInfo;
        return _regenerator["default"].wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                _context7.next = 2;
                return this.getKey(keyId);

              case 2:
                keyInfo = _context7.sent;

                if (!(keyInfo && keyInfo[1].algorithm === SECRET_STORAGE_ALGORITHM_V1_CURVE25519)) {
                  _context7.next = 7;
                  break;
                }

                return _context7.abrupt("return", true);

              case 7:
                return _context7.abrupt("return", false);

              case 8:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function keyNeedsUpgrade(_x9) {
        return _keyNeedsUpgrade.apply(this, arguments);
      }

      return keyNeedsUpgrade;
    }()
    /**
     * Check whether a key matches what we expect based on the key info
     *
     * @param {Uint8Array} key the key to check
     * @param {object} info the key info
     *
     * @return {boolean} whether or not the key matches
     */

  }, {
    key: "checkKey",
    value: function () {
      var _checkKey = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee8(key, info) {
        var _ref3, mac, decryption, gotPubkey;

        return _regenerator["default"].wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                _context8.t0 = info.algorithm;
                _context8.next = _context8.t0 === SECRET_STORAGE_ALGORITHM_V1_AES ? 3 : _context8.t0 === SECRET_STORAGE_ALGORITHM_V1_CURVE25519 ? 12 : 25;
                break;

              case 3:
                if (!info.mac) {
                  _context8.next = 11;
                  break;
                }

                _context8.next = 6;
                return SecretStorage._calculateKeyCheck(key, info.iv);

              case 6:
                _ref3 = _context8.sent;
                mac = _ref3.mac;
                return _context8.abrupt("return", info.mac === mac);

              case 11:
                return _context8.abrupt("return", true);

              case 12:
                decryption = null;
                _context8.prev = 13;
                decryption = new global.Olm.PkDecryption();
                gotPubkey = decryption.init_with_private_key(key); // make sure it agrees with the given pubkey

                return _context8.abrupt("return", gotPubkey === info.pubkey);

              case 19:
                _context8.prev = 19;
                _context8.t1 = _context8["catch"](13);
                return _context8.abrupt("return", false);

              case 22:
                _context8.prev = 22;
                if (decryption) decryption.free();
                return _context8.finish(22);

              case 25:
                throw new Error("Unknown algorithm");

              case 26:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, null, [[13, 19, 22, 25]]);
      }));

      function checkKey(_x10, _x11) {
        return _checkKey.apply(this, arguments);
      }

      return checkKey;
    }()
  }, {
    key: "store",

    /**
     * Store an encrypted secret on the server
     *
     * @param {string} name The name of the secret
     * @param {string} secret The secret contents.
     * @param {Array} keys The IDs of the keys to use to encrypt the secret
     *     or null/undefined to use the default key.
     */
    value: function () {
      var _store = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee9(name, secret, keys) {
        var encrypted, defaultKeyId, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, keyId, keyInfo, _keys, _ref4, _ref5, encryption;

        return _regenerator["default"].wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                encrypted = {};

                if (keys) {
                  _context9.next = 8;
                  break;
                }

                _context9.next = 4;
                return this.getDefaultKeyId();

              case 4:
                defaultKeyId = _context9.sent;

                if (defaultKeyId) {
                  _context9.next = 7;
                  break;
                }

                throw new Error("No keys specified and no default key present");

              case 7:
                keys = [defaultKeyId];

              case 8:
                if (!(keys.length === 0)) {
                  _context9.next = 10;
                  break;
                }

                throw new Error("Zero keys given to encrypt with!");

              case 10:
                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context9.prev = 13;
                _iterator = keys[Symbol.iterator]();

              case 15:
                if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                  _context9.next = 39;
                  break;
                }

                keyId = _step.value;
                _context9.next = 19;
                return this._baseApis.getAccountDataFromServer("m.secret_storage.key." + keyId);

              case 19:
                keyInfo = _context9.sent;

                if (keyInfo) {
                  _context9.next = 22;
                  break;
                }

                throw new Error("Unknown key: " + keyId);

              case 22:
                _context9.t0 = keyInfo.algorithm;
                _context9.next = _context9.t0 === SECRET_STORAGE_ALGORITHM_V1_AES ? 25 : 35;
                break;

              case 25:
                _keys = (0, _defineProperty2["default"])({}, keyId, keyInfo);
                _context9.next = 28;
                return this._getSecretStorageKey(_keys, name);

              case 28:
                _ref4 = _context9.sent;
                _ref5 = (0, _slicedToArray2["default"])(_ref4, 2);
                encryption = _ref5[1];
                _context9.next = 33;
                return encryption.encrypt(secret);

              case 33:
                encrypted[keyId] = _context9.sent;
                return _context9.abrupt("break", 36);

              case 35:
                _logger.logger.warn("unknown algorithm for secret storage key " + keyId + ": " + keyInfo.algorithm);

              case 36:
                _iteratorNormalCompletion = true;
                _context9.next = 15;
                break;

              case 39:
                _context9.next = 45;
                break;

              case 41:
                _context9.prev = 41;
                _context9.t1 = _context9["catch"](13);
                _didIteratorError = true;
                _iteratorError = _context9.t1;

              case 45:
                _context9.prev = 45;
                _context9.prev = 46;

                if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                  _iterator["return"]();
                }

              case 48:
                _context9.prev = 48;

                if (!_didIteratorError) {
                  _context9.next = 51;
                  break;
                }

                throw _iteratorError;

              case 51:
                return _context9.finish(48);

              case 52:
                return _context9.finish(45);

              case 53:
                _context9.next = 55;
                return this._baseApis.setAccountData(name, {
                  encrypted: encrypted
                });

              case 55:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this, [[13, 41, 45, 53], [46,, 48, 52]]);
      }));

      function store(_x12, _x13, _x14) {
        return _store.apply(this, arguments);
      }

      return store;
    }()
    /**
     * Temporary method to fix up existing accounts where secrets
     * are incorrectly stored without the 'encrypted' level
     *
     * @param {string} name The name of the secret
     * @param {object} secretInfo The account data object
     * @returns {object} The fixed object or null if no fix was performed
     */

  }, {
    key: "_fixupStoredSecret",
    value: function () {
      var _fixupStoredSecret2 = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee10(name, secretInfo) {
        var keys, hasKey, newData;
        return _regenerator["default"].wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                // We assume the secret was only stored passthrough for 1
                // key - this was all the broken code supported.
                keys = Object.keys(secretInfo);

                if (!(keys.length === 1 && keys[0] !== 'encrypted' && secretInfo[keys[0]].passthrough)) {
                  _context10.next = 13;
                  break;
                }

                _context10.next = 4;
                return this.hasKey(keys[0]);

              case 4:
                hasKey = _context10.sent;

                if (!hasKey) {
                  _context10.next = 13;
                  break;
                }

                console.log("Fixing up passthrough secret: " + name);
                _context10.next = 9;
                return this.storePassthrough(name, keys[0]);

              case 9:
                _context10.next = 11;
                return this._baseApis.getAccountDataFromServer(name);

              case 11:
                newData = _context10.sent;
                return _context10.abrupt("return", newData);

              case 13:
                return _context10.abrupt("return", null);

              case 14:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));

      function _fixupStoredSecret(_x15, _x16) {
        return _fixupStoredSecret2.apply(this, arguments);
      }

      return _fixupStoredSecret;
    }()
    /**
     * Get a secret from storage.
     *
     * @param {string} name the name of the secret
     *
     * @return {string} the contents of the secret
     */

  }, {
    key: "get",
    value: function () {
      var _get = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee11(name) {
        var secretInfo, keys, _i, _Object$keys, _keyId, keyInfo, _encInfo, keyId, decryption, _ref6, _ref7, encInfo;

        return _regenerator["default"].wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                _context11.next = 2;
                return this._baseApis.getAccountDataFromServer(name);

              case 2:
                secretInfo = _context11.sent;

                if (secretInfo) {
                  _context11.next = 5;
                  break;
                }

                return _context11.abrupt("return");

              case 5:
                if (secretInfo.encrypted) {
                  _context11.next = 11;
                  break;
                }

                _context11.next = 8;
                return this._fixupStoredSecret(name, secretInfo);

              case 8:
                secretInfo = _context11.sent;

                if (!(!secretInfo || !secretInfo.encrypted)) {
                  _context11.next = 11;
                  break;
                }

                throw new Error("Content is not encrypted!");

              case 11:
                // get possible keys to decrypt
                keys = {};
                _i = 0, _Object$keys = Object.keys(secretInfo.encrypted);

              case 13:
                if (!(_i < _Object$keys.length)) {
                  _context11.next = 29;
                  break;
                }

                _keyId = _Object$keys[_i];
                _context11.next = 17;
                return this._baseApis.getAccountDataFromServer("m.secret_storage.key." + _keyId);

              case 17:
                keyInfo = _context11.sent;
                _encInfo = secretInfo.encrypted[_keyId];
                _context11.t0 = keyInfo.algorithm;
                _context11.next = _context11.t0 === SECRET_STORAGE_ALGORITHM_V1_AES ? 22 : _context11.t0 === SECRET_STORAGE_ALGORITHM_V1_CURVE25519 ? 24 : 26;
                break;

              case 22:
                if (_encInfo.iv && _encInfo.ciphertext && _encInfo.mac) {
                  keys[_keyId] = keyInfo;
                }

                return _context11.abrupt("break", 26);

              case 24:
                if (keyInfo.pubkey && (_encInfo.ciphertext && _encInfo.mac && _encInfo.ephemeral || _encInfo.passthrough)) {
                  keys[_keyId] = keyInfo;
                }

                return _context11.abrupt("break", 26);

              case 26:
                _i++;
                _context11.next = 13;
                break;

              case 29:
                _context11.prev = 29;
                _context11.next = 32;
                return this._getSecretStorageKey(keys, name);

              case 32:
                _ref6 = _context11.sent;
                _ref7 = (0, _slicedToArray2["default"])(_ref6, 2);
                keyId = _ref7[0];
                decryption = _ref7[1];
                encInfo = secretInfo.encrypted[keyId]; // We don't actually need the decryption object if it's a passthrough
                // since we just want to return the key itself. It must be base64
                // encoded, since this is how a key would normally be stored.

                if (!encInfo.passthrough) {
                  _context11.next = 39;
                  break;
                }

                return _context11.abrupt("return", (0, olmlib.encodeBase64)(decryption.get_private_key()));

              case 39:
                _context11.next = 41;
                return decryption.decrypt(encInfo);

              case 41:
                return _context11.abrupt("return", _context11.sent);

              case 42:
                _context11.prev = 42;
                if (decryption && decryption.free) decryption.free();
                return _context11.finish(42);

              case 45:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11, this, [[29,, 42, 45]]);
      }));

      function get(_x17) {
        return _get.apply(this, arguments);
      }

      return get;
    }()
    /**
     * Check if a secret is stored on the server.
     *
     * @param {string} name the name of the secret
     * @param {boolean} checkKey check if the secret is encrypted by a trusted key
     *
     * @return {object?} map of key name to key info the secret is encrypted
     *     with, or null if it is not present or not encrypted with a trusted
     *     key
     */

  }, {
    key: "isStored",
    value: function () {
      var _isStored = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee12(name, checkKey) {
        var secretInfo, ret, _i2, _Object$keys2, keyId, keyInfo, encInfo;

        return _regenerator["default"].wrap(function _callee12$(_context12) {
          while (1) {
            switch (_context12.prev = _context12.next) {
              case 0:
                _context12.next = 2;
                return this._baseApis.getAccountDataFromServer(name);

              case 2:
                secretInfo = _context12.sent;

                if (secretInfo) {
                  _context12.next = 5;
                  break;
                }

                return _context12.abrupt("return", null);

              case 5:
                if (secretInfo.encrypted) {
                  _context12.next = 11;
                  break;
                }

                _context12.next = 8;
                return this._fixupStoredSecret(name, secretInfo);

              case 8:
                secretInfo = _context12.sent;

                if (!(!secretInfo || !secretInfo.encrypted)) {
                  _context12.next = 11;
                  break;
                }

                return _context12.abrupt("return", null);

              case 11:
                if (checkKey === undefined) checkKey = true;
                ret = {}; // check if secret is encrypted by a known/trusted secret and
                // encryption looks sane

                _i2 = 0, _Object$keys2 = Object.keys(secretInfo.encrypted);

              case 14:
                if (!(_i2 < _Object$keys2.length)) {
                  _context12.next = 51;
                  break;
                }

                keyId = _Object$keys2[_i2];
                _context12.next = 18;
                return this._baseApis.getAccountDataFromServer("m.secret_storage.key." + keyId);

              case 18:
                keyInfo = _context12.sent;

                if (keyInfo) {
                  _context12.next = 21;
                  break;
                }

                return _context12.abrupt("continue", 48);

              case 21:
                encInfo = secretInfo.encrypted[keyId]; // We don't actually need the decryption object if it's a passthrough
                // since we just want to return the key itself.

                if (!encInfo.passthrough) {
                  _context12.next = 32;
                  break;
                }

                _context12.prev = 23;
                (0, olmlib.pkVerify)(keyInfo, this._crossSigningInfo.getId('master'), this._crossSigningInfo.userId);
                _context12.next = 30;
                break;

              case 27:
                _context12.prev = 27;
                _context12.t0 = _context12["catch"](23);
                return _context12.abrupt("continue", 48);

              case 30:
                ret[keyId] = keyInfo;
                return _context12.abrupt("continue", 48);

              case 32:
                _context12.t1 = keyInfo.algorithm;
                _context12.next = _context12.t1 === SECRET_STORAGE_ALGORITHM_V1_AES ? 35 : _context12.t1 === SECRET_STORAGE_ALGORITHM_V1_CURVE25519 ? 37 : 48;
                break;

              case 35:
                if (encInfo.iv && encInfo.ciphertext && encInfo.mac) {
                  ret[keyId] = keyInfo;
                }

                return _context12.abrupt("break", 48);

              case 37:
                if (!(keyInfo.pubkey && encInfo.ciphertext && encInfo.mac && encInfo.ephemeral)) {
                  _context12.next = 47;
                  break;
                }

                if (!checkKey) {
                  _context12.next = 46;
                  break;
                }

                _context12.prev = 39;
                (0, olmlib.pkVerify)(keyInfo, this._crossSigningInfo.getId('master'), this._crossSigningInfo.userId);
                _context12.next = 46;
                break;

              case 43:
                _context12.prev = 43;
                _context12.t2 = _context12["catch"](39);
                return _context12.abrupt("continue", 48);

              case 46:
                ret[keyId] = keyInfo;

              case 47:
                return _context12.abrupt("break", 48);

              case 48:
                _i2++;
                _context12.next = 14;
                break;

              case 51:
                return _context12.abrupt("return", Object.keys(ret).length ? ret : null);

              case 52:
              case "end":
                return _context12.stop();
            }
          }
        }, _callee12, this, [[23, 27], [39, 43]]);
      }));

      function isStored(_x18, _x19) {
        return _isStored.apply(this, arguments);
      }

      return isStored;
    }()
    /**
     * Request a secret from another device
     *
     * @param {string} name the name of the secret to request
     * @param {string[]} devices the devices to request the secret from
     *
     * @return {string} the contents of the secret
     */

  }, {
    key: "request",
    value: function request(name, devices) {
      var _this3 = this;

      var requestId = this._baseApis.makeTxnId();

      var requestControl = this._requests[requestId] = {
        devices: devices
      };
      var promise = new Promise(function (resolve, reject) {
        requestControl.resolve = resolve;
        requestControl.reject = reject;
      });

      var cancel = function cancel(reason) {
        // send cancellation event
        var cancelData = {
          action: "request_cancellation",
          requesting_device_id: _this3._baseApis.deviceId,
          request_id: requestId
        };
        var toDevice = {};
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = devices[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var device = _step2.value;
            toDevice[device] = cancelData;
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
              _iterator2["return"]();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        _this3._baseApis.sendToDevice("m.secret.request", (0, _defineProperty2["default"])({}, _this3._baseApis.getUserId(), toDevice)); // and reject the promise so that anyone waiting on it will be
        // notified


        requestControl.reject(new Error(reason || "Cancelled"));
      }; // send request to devices


      var requestData = {
        name: name,
        action: "request",
        requesting_device_id: this._baseApis.deviceId,
        request_id: requestId
      };
      var toDevice = {};
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = devices[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var device = _step3.value;
          toDevice[device] = requestData;
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3["return"] != null) {
            _iterator3["return"]();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      _logger.logger.info("Request secret ".concat(name, " from ").concat(devices, ", id ").concat(requestId));

      this._baseApis.sendToDevice("m.secret.request", (0, _defineProperty2["default"])({}, this._baseApis.getUserId(), toDevice));

      return {
        request_id: requestId,
        promise: promise,
        cancel: cancel
      };
    }
  }, {
    key: "_onRequestReceived",
    value: function () {
      var _onRequestReceived2 = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee13(event) {
        var sender, content, deviceId, secret, payload, encryptedContent, contentMap;
        return _regenerator["default"].wrap(function _callee13$(_context13) {
          while (1) {
            switch (_context13.prev = _context13.next) {
              case 0:
                sender = event.getSender();
                content = event.getContent();

                if (!(sender !== this._baseApis.getUserId() || !(content.name && content.action && content.requesting_device_id && content.request_id))) {
                  _context13.next = 4;
                  break;
                }

                return _context13.abrupt("return");

              case 4:
                deviceId = content.requesting_device_id; // check if it's a cancel

                if (!(content.action === "request_cancellation")) {
                  _context13.next = 9;
                  break;
                }

                if (this._incomingRequests[deviceId] && this._incomingRequests[deviceId][content.request_id]) {
                  _logger.logger.info("received request cancellation for secret (" + sender + ", " + deviceId + ", " + content.request_id + ")");

                  this.baseApis.emit("crypto.secrets.requestCancelled", {
                    user_id: sender,
                    device_id: deviceId,
                    request_id: content.request_id
                  });
                }

                _context13.next = 32;
                break;

              case 9:
                if (!(content.action === "request")) {
                  _context13.next = 32;
                  break;
                }

                if (!(deviceId === this._baseApis.deviceId)) {
                  _context13.next = 12;
                  break;
                }

                return _context13.abrupt("return");

              case 12:
                // check if we have the secret
                _logger.logger.info("received request for secret (" + sender + ", " + deviceId + ", " + content.request_id + ")");

                if (this._cryptoCallbacks.onSecretRequested) {
                  _context13.next = 15;
                  break;
                }

                return _context13.abrupt("return");

              case 15:
                _context13.next = 17;
                return this._cryptoCallbacks.onSecretRequested({
                  user_id: sender,
                  device_id: deviceId,
                  request_id: content.request_id,
                  name: content.name,
                  device_trust: this._baseApis.checkDeviceTrust(sender, deviceId)
                });

              case 17:
                secret = _context13.sent;

                if (!secret) {
                  _context13.next = 31;
                  break;
                }

                _logger.logger.info("Preparing ".concat(content.name, " secret for ").concat(deviceId));

                payload = {
                  type: "m.secret.send",
                  content: {
                    request_id: content.request_id,
                    secret: secret
                  }
                };
                encryptedContent = {
                  algorithm: olmlib.OLM_ALGORITHM,
                  sender_key: this._baseApis._crypto._olmDevice.deviceCurve25519Key,
                  ciphertext: {}
                };
                _context13.next = 24;
                return olmlib.ensureOlmSessionsForDevices(this._baseApis._crypto._olmDevice, this._baseApis, (0, _defineProperty2["default"])({}, sender, [this._baseApis.getStoredDevice(sender, deviceId)]));

              case 24:
                _context13.next = 26;
                return olmlib.encryptMessageForDevice(encryptedContent.ciphertext, this._baseApis.getUserId(), this._baseApis.deviceId, this._baseApis._crypto._olmDevice, sender, this._baseApis.getStoredDevice(sender, deviceId), payload);

              case 26:
                contentMap = (0, _defineProperty2["default"])({}, sender, (0, _defineProperty2["default"])({}, deviceId, encryptedContent));

                _logger.logger.info("Sending ".concat(content.name, " secret for ").concat(deviceId));

                this._baseApis.sendToDevice("m.room.encrypted", contentMap);

                _context13.next = 32;
                break;

              case 31:
                _logger.logger.info("Request denied for ".concat(content.name, " secret for ").concat(deviceId));

              case 32:
              case "end":
                return _context13.stop();
            }
          }
        }, _callee13, this);
      }));

      function _onRequestReceived(_x20) {
        return _onRequestReceived2.apply(this, arguments);
      }

      return _onRequestReceived;
    }()
  }, {
    key: "_onSecretReceived",
    value: function _onSecretReceived(event) {
      if (event.getSender() !== this._baseApis.getUserId()) {
        // we shouldn't be receiving secrets from anyone else, so ignore
        // because someone could be trying to send us bogus data
        return;
      }

      var content = event.getContent();

      _logger.logger.log("got secret share for request", content.request_id);

      var requestControl = this._requests[content.request_id];

      if (requestControl) {
        // make sure that the device that sent it is one of the devices that
        // we requested from
        var deviceInfo = this._baseApis._crypto._deviceList.getDeviceByIdentityKey(olmlib.OLM_ALGORITHM, event.getSenderKey());

        if (!deviceInfo) {
          _logger.logger.log("secret share from unknown device with key", event.getSenderKey());

          return;
        }

        if (!requestControl.devices.includes(deviceInfo.deviceId)) {
          _logger.logger.log("unsolicited secret share from device", deviceInfo.deviceId);

          return;
        }

        requestControl.resolve(content.secret);
      }
    }
  }, {
    key: "_getSecretStorageKey",
    value: function () {
      var _getSecretStorageKey2 = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee17(keys, name) {
        var returned, _returned, keyId, privateKey, decryption, pkDecryption, pubkey, _decryption;

        return _regenerator["default"].wrap(function _callee17$(_context17) {
          while (1) {
            switch (_context17.prev = _context17.next) {
              case 0:
                if (this._cryptoCallbacks.getSecretStorageKey) {
                  _context17.next = 2;
                  break;
                }

                throw new Error("No getSecretStorageKey callback supplied");

              case 2:
                _context17.next = 4;
                return this._cryptoCallbacks.getSecretStorageKey({
                  keys: keys
                }, name);

              case 4:
                returned = _context17.sent;

                if (returned) {
                  _context17.next = 7;
                  break;
                }

                throw new Error("getSecretStorageKey callback returned falsey");

              case 7:
                if (!(returned.length < 2)) {
                  _context17.next = 9;
                  break;
                }

                throw new Error("getSecretStorageKey callback returned invalid data");

              case 9:
                _returned = (0, _slicedToArray2["default"])(returned, 2), keyId = _returned[0], privateKey = _returned[1];

                if (keys[keyId]) {
                  _context17.next = 12;
                  break;
                }

                throw new Error("App returned unknown key from getSecretStorageKey!");

              case 12:
                _context17.t0 = keys[keyId].algorithm;
                _context17.next = _context17.t0 === SECRET_STORAGE_ALGORITHM_V1_AES ? 15 : _context17.t0 === SECRET_STORAGE_ALGORITHM_V1_CURVE25519 ? 17 : 31;
                break;

              case 15:
                decryption = {
                  encrypt: function () {
                    var _encrypt = (0, _asyncToGenerator2["default"])(
                    /*#__PURE__*/
                    _regenerator["default"].mark(function _callee14(secret) {
                      return _regenerator["default"].wrap(function _callee14$(_context14) {
                        while (1) {
                          switch (_context14.prev = _context14.next) {
                            case 0:
                              _context14.next = 2;
                              return (0, _aes.encryptAES)(secret, privateKey, name);

                            case 2:
                              return _context14.abrupt("return", _context14.sent);

                            case 3:
                            case "end":
                              return _context14.stop();
                          }
                        }
                      }, _callee14);
                    }));

                    function encrypt(_x23) {
                      return _encrypt.apply(this, arguments);
                    }

                    return encrypt;
                  }(),
                  decrypt: function () {
                    var _decrypt = (0, _asyncToGenerator2["default"])(
                    /*#__PURE__*/
                    _regenerator["default"].mark(function _callee15(encInfo) {
                      return _regenerator["default"].wrap(function _callee15$(_context15) {
                        while (1) {
                          switch (_context15.prev = _context15.next) {
                            case 0:
                              _context15.next = 2;
                              return (0, _aes.decryptAES)(encInfo, privateKey, name);

                            case 2:
                              return _context15.abrupt("return", _context15.sent);

                            case 3:
                            case "end":
                              return _context15.stop();
                          }
                        }
                      }, _callee15);
                    }));

                    function decrypt(_x24) {
                      return _decrypt.apply(this, arguments);
                    }

                    return decrypt;
                  }()
                };
                return _context17.abrupt("return", [keyId, decryption]);

              case 17:
                pkDecryption = new global.Olm.PkDecryption();
                _context17.prev = 18;
                pubkey = pkDecryption.init_with_private_key(privateKey);
                _context17.next = 26;
                break;

              case 22:
                _context17.prev = 22;
                _context17.t1 = _context17["catch"](18);
                pkDecryption.free();
                throw new Error("getSecretStorageKey callback returned invalid key");

              case 26:
                if (!(pubkey !== keys[keyId].pubkey)) {
                  _context17.next = 29;
                  break;
                }

                pkDecryption.free();
                throw new Error("getSecretStorageKey callback returned incorrect key");

              case 29:
                _decryption = {
                  free: pkDecryption.free.bind(pkDecryption),
                  decrypt: function () {
                    var _decrypt2 = (0, _asyncToGenerator2["default"])(
                    /*#__PURE__*/
                    _regenerator["default"].mark(function _callee16(encInfo) {
                      return _regenerator["default"].wrap(function _callee16$(_context16) {
                        while (1) {
                          switch (_context16.prev = _context16.next) {
                            case 0:
                              return _context16.abrupt("return", pkDecryption.decrypt(encInfo.ephemeral, encInfo.mac, encInfo.ciphertext));

                            case 1:
                            case "end":
                              return _context16.stop();
                          }
                        }
                      }, _callee16);
                    }));

                    function decrypt(_x25) {
                      return _decrypt2.apply(this, arguments);
                    }

                    return decrypt;
                  }(),
                  // needed for passthrough
                  get_private_key: pkDecryption.get_private_key.bind(pkDecryption)
                };
                return _context17.abrupt("return", [keyId, _decryption]);

              case 31:
                throw new Error("Unknown key type: " + keys[keyId].algorithm);

              case 32:
              case "end":
                return _context17.stop();
            }
          }
        }, _callee17, this, [[18, 22]]);
      }));

      function _getSecretStorageKey(_x21, _x22) {
        return _getSecretStorageKey2.apply(this, arguments);
      }

      return _getSecretStorageKey;
    }()
  }], [{
    key: "_calculateKeyCheck",
    value: function () {
      var _calculateKeyCheck2 = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee18(key, iv) {
        return _regenerator["default"].wrap(function _callee18$(_context18) {
          while (1) {
            switch (_context18.prev = _context18.next) {
              case 0:
                _context18.next = 2;
                return (0, _aes.encryptAES)(ZERO_STR, key, "", iv);

              case 2:
                return _context18.abrupt("return", _context18.sent);

              case 3:
              case "end":
                return _context18.stop();
            }
          }
        }, _callee18);
      }));

      function _calculateKeyCheck(_x26, _x27) {
        return _calculateKeyCheck2.apply(this, arguments);
      }

      return _calculateKeyCheck;
    }()
  }]);
  return SecretStorage;
}(_events.EventEmitter);

exports.SecretStorage = SecretStorage;