"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LocalStorageCryptoStore = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _logger = require("../../logger");

var _memoryCryptoStore = require("./memory-crypto-store");

/*
Copyright 2017, 2018 New Vector Ltd
Copyright 2020 The Matrix.org Foundation C.I.C.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Internal module. Partial localStorage backed storage for e2e.
 * This is not a full crypto store, just the in-memory store with
 * some things backed by localStorage. It exists because indexedDB
 * is broken in Firefox private mode or set to, "will not remember
 * history".
 *
 * @module
 */
var E2E_PREFIX = "crypto.";
var KEY_END_TO_END_ACCOUNT = E2E_PREFIX + "account";
var KEY_CROSS_SIGNING_KEYS = E2E_PREFIX + "cross_signing_keys";
var KEY_NOTIFIED_ERROR_DEVICES = E2E_PREFIX + "notified_error_devices";
var KEY_DEVICE_DATA = E2E_PREFIX + "device_data";
var KEY_INBOUND_SESSION_PREFIX = E2E_PREFIX + "inboundgroupsessions/";
var KEY_INBOUND_SESSION_WITHHELD_PREFIX = E2E_PREFIX + "inboundgroupsessions.withheld/";
var KEY_ROOMS_PREFIX = E2E_PREFIX + "rooms/";
var KEY_SESSIONS_NEEDING_BACKUP = E2E_PREFIX + "sessionsneedingbackup";

function keyEndToEndSessions(deviceKey) {
  return E2E_PREFIX + "sessions/" + deviceKey;
}

function keyEndToEndSessionProblems(deviceKey) {
  return E2E_PREFIX + "session.problems/" + deviceKey;
}

function keyEndToEndInboundGroupSession(senderKey, sessionId) {
  return KEY_INBOUND_SESSION_PREFIX + senderKey + "/" + sessionId;
}

function keyEndToEndInboundGroupSessionWithheld(senderKey, sessionId) {
  return KEY_INBOUND_SESSION_WITHHELD_PREFIX + senderKey + "/" + sessionId;
}

function keyEndToEndRoomsPrefix(roomId) {
  return KEY_ROOMS_PREFIX + roomId;
}
/**
 * @implements {module:crypto/store/base~CryptoStore}
 */


var LocalStorageCryptoStore =
/*#__PURE__*/
function (_MemoryCryptoStore) {
  (0, _inherits2["default"])(LocalStorageCryptoStore, _MemoryCryptoStore);

  function LocalStorageCryptoStore(webStore) {
    var _this;

    (0, _classCallCheck2["default"])(this, LocalStorageCryptoStore);
    _this = (0, _possibleConstructorReturn2["default"])(this, (0, _getPrototypeOf2["default"])(LocalStorageCryptoStore).call(this));
    _this.store = webStore;
    return _this;
  }

  (0, _createClass2["default"])(LocalStorageCryptoStore, [{
    key: "countEndToEndSessions",
    // Olm Sessions
    value: function countEndToEndSessions(txn, func) {
      var count = 0;

      for (var i = 0; i < this.store.length; ++i) {
        if (this.store.key(i).startsWith(keyEndToEndSessions(''))) ++count;
      }

      func(count);
    }
  }, {
    key: "_getEndToEndSessions",
    value: function _getEndToEndSessions(deviceKey, txn, func) {
      var sessions = getJsonItem(this.store, keyEndToEndSessions(deviceKey));
      var fixedSessions = {}; // fix up any old sessions to be objects rather than just the base64 pickle

      for (var _i = 0, _Object$entries = Object.entries(sessions || {}); _i < _Object$entries.length; _i++) {
        var _Object$entries$_i = (0, _slicedToArray2["default"])(_Object$entries[_i], 2),
            sid = _Object$entries$_i[0],
            val = _Object$entries$_i[1];

        if (typeof val === 'string') {
          fixedSessions[sid] = {
            session: val
          };
        } else {
          fixedSessions[sid] = val;
        }
      }

      return fixedSessions;
    }
  }, {
    key: "getEndToEndSession",
    value: function getEndToEndSession(deviceKey, sessionId, txn, func) {
      var sessions = this._getEndToEndSessions(deviceKey);

      func(sessions[sessionId] || {});
    }
  }, {
    key: "getEndToEndSessions",
    value: function getEndToEndSessions(deviceKey, txn, func) {
      func(this._getEndToEndSessions(deviceKey) || {});
    }
  }, {
    key: "getAllEndToEndSessions",
    value: function getAllEndToEndSessions(txn, func) {
      for (var i = 0; i < this.store.length; ++i) {
        if (this.store.key(i).startsWith(keyEndToEndSessions(''))) {
          var deviceKey = this.store.key(i).split('/')[1];

          for (var _i2 = 0, _Object$values = Object.values(this._getEndToEndSessions(deviceKey)); _i2 < _Object$values.length; _i2++) {
            var sess = _Object$values[_i2];
            func(sess);
          }
        }
      }
    }
  }, {
    key: "storeEndToEndSession",
    value: function storeEndToEndSession(deviceKey, sessionId, sessionInfo, txn) {
      var sessions = this._getEndToEndSessions(deviceKey) || {};
      sessions[sessionId] = sessionInfo;
      setJsonItem(this.store, keyEndToEndSessions(deviceKey), sessions);
    }
  }, {
    key: "storeEndToEndSessionProblem",
    value: function () {
      var _storeEndToEndSessionProblem = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee(deviceKey, type, fixed) {
        var key, problems;
        return _regenerator["default"].wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                key = keyEndToEndSessionProblems(deviceKey);
                problems = getJsonItem(this.store, key) || [];
                problems.push({
                  type: type,
                  fixed: fixed,
                  time: Date.now()
                });
                problems.sort(function (a, b) {
                  return a.time - b.time;
                });
                setJsonItem(this.store, key, problems);

              case 5:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function storeEndToEndSessionProblem(_x, _x2, _x3) {
        return _storeEndToEndSessionProblem.apply(this, arguments);
      }

      return storeEndToEndSessionProblem;
    }()
  }, {
    key: "getEndToEndSessionProblem",
    value: function () {
      var _getEndToEndSessionProblem = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee2(deviceKey, timestamp) {
        var key, problems, lastProblem, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, problem;

        return _regenerator["default"].wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                key = keyEndToEndSessionProblems(deviceKey);
                problems = getJsonItem(this.store, key) || [];

                if (problems.length) {
                  _context2.next = 4;
                  break;
                }

                return _context2.abrupt("return", null);

              case 4:
                lastProblem = problems[problems.length - 1];
                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context2.prev = 8;
                _iterator = problems[Symbol.iterator]();

              case 10:
                if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                  _context2.next = 17;
                  break;
                }

                problem = _step.value;

                if (!(problem.time > timestamp)) {
                  _context2.next = 14;
                  break;
                }

                return _context2.abrupt("return", Object.assign({}, problem, {
                  fixed: lastProblem.fixed
                }));

              case 14:
                _iteratorNormalCompletion = true;
                _context2.next = 10;
                break;

              case 17:
                _context2.next = 23;
                break;

              case 19:
                _context2.prev = 19;
                _context2.t0 = _context2["catch"](8);
                _didIteratorError = true;
                _iteratorError = _context2.t0;

              case 23:
                _context2.prev = 23;
                _context2.prev = 24;

                if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                  _iterator["return"]();
                }

              case 26:
                _context2.prev = 26;

                if (!_didIteratorError) {
                  _context2.next = 29;
                  break;
                }

                throw _iteratorError;

              case 29:
                return _context2.finish(26);

              case 30:
                return _context2.finish(23);

              case 31:
                if (!lastProblem.fixed) {
                  _context2.next = 35;
                  break;
                }

                return _context2.abrupt("return", null);

              case 35:
                return _context2.abrupt("return", lastProblem);

              case 36:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this, [[8, 19, 23, 31], [24,, 26, 30]]);
      }));

      function getEndToEndSessionProblem(_x4, _x5) {
        return _getEndToEndSessionProblem.apply(this, arguments);
      }

      return getEndToEndSessionProblem;
    }()
  }, {
    key: "filterOutNotifiedErrorDevices",
    value: function () {
      var _filterOutNotifiedErrorDevices = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee3(devices) {
        var notifiedErrorDevices, ret, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, device, userId, deviceInfo;

        return _regenerator["default"].wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                notifiedErrorDevices = getJsonItem(this.store, KEY_NOTIFIED_ERROR_DEVICES) || {};
                ret = [];
                _iteratorNormalCompletion2 = true;
                _didIteratorError2 = false;
                _iteratorError2 = undefined;
                _context3.prev = 5;

                for (_iterator2 = devices[Symbol.iterator](); !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                  device = _step2.value;
                  userId = device.userId, deviceInfo = device.deviceInfo;

                  if (userId in notifiedErrorDevices) {
                    if (!(deviceInfo.deviceId in notifiedErrorDevices[userId])) {
                      ret.push(device);
                      notifiedErrorDevices[userId][deviceInfo.deviceId] = true;
                    }
                  } else {
                    ret.push(device);
                    notifiedErrorDevices[userId] = (0, _defineProperty2["default"])({}, deviceInfo.deviceId, true);
                  }
                }

                _context3.next = 13;
                break;

              case 9:
                _context3.prev = 9;
                _context3.t0 = _context3["catch"](5);
                _didIteratorError2 = true;
                _iteratorError2 = _context3.t0;

              case 13:
                _context3.prev = 13;
                _context3.prev = 14;

                if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
                  _iterator2["return"]();
                }

              case 16:
                _context3.prev = 16;

                if (!_didIteratorError2) {
                  _context3.next = 19;
                  break;
                }

                throw _iteratorError2;

              case 19:
                return _context3.finish(16);

              case 20:
                return _context3.finish(13);

              case 21:
                setJsonItem(this.store, KEY_NOTIFIED_ERROR_DEVICES, notifiedErrorDevices);
                return _context3.abrupt("return", ret);

              case 23:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this, [[5, 9, 13, 21], [14,, 16, 20]]);
      }));

      function filterOutNotifiedErrorDevices(_x6) {
        return _filterOutNotifiedErrorDevices.apply(this, arguments);
      }

      return filterOutNotifiedErrorDevices;
    }() // Inbound Group Sessions

  }, {
    key: "getEndToEndInboundGroupSession",
    value: function getEndToEndInboundGroupSession(senderCurve25519Key, sessionId, txn, func) {
      func(getJsonItem(this.store, keyEndToEndInboundGroupSession(senderCurve25519Key, sessionId)), getJsonItem(this.store, keyEndToEndInboundGroupSessionWithheld(senderCurve25519Key, sessionId)));
    }
  }, {
    key: "getAllEndToEndInboundGroupSessions",
    value: function getAllEndToEndInboundGroupSessions(txn, func) {
      for (var i = 0; i < this.store.length; ++i) {
        var key = this.store.key(i);

        if (key.startsWith(KEY_INBOUND_SESSION_PREFIX)) {
          // we can't use split, as the components we are trying to split out
          // might themselves contain '/' characters. We rely on the
          // senderKey being a (32-byte) curve25519 key, base64-encoded
          // (hence 43 characters long).
          func({
            senderKey: key.substr(KEY_INBOUND_SESSION_PREFIX.length, 43),
            sessionId: key.substr(KEY_INBOUND_SESSION_PREFIX.length + 44),
            sessionData: getJsonItem(this.store, key)
          });
        }
      }

      func(null);
    }
  }, {
    key: "addEndToEndInboundGroupSession",
    value: function addEndToEndInboundGroupSession(senderCurve25519Key, sessionId, sessionData, txn) {
      var existing = getJsonItem(this.store, keyEndToEndInboundGroupSession(senderCurve25519Key, sessionId));

      if (!existing) {
        this.storeEndToEndInboundGroupSession(senderCurve25519Key, sessionId, sessionData, txn);
      }
    }
  }, {
    key: "storeEndToEndInboundGroupSession",
    value: function storeEndToEndInboundGroupSession(senderCurve25519Key, sessionId, sessionData, txn) {
      setJsonItem(this.store, keyEndToEndInboundGroupSession(senderCurve25519Key, sessionId), sessionData);
    }
  }, {
    key: "storeEndToEndInboundGroupSessionWithheld",
    value: function storeEndToEndInboundGroupSessionWithheld(senderCurve25519Key, sessionId, sessionData, txn) {
      setJsonItem(this.store, keyEndToEndInboundGroupSessionWithheld(senderCurve25519Key, sessionId), sessionData);
    }
  }, {
    key: "getEndToEndDeviceData",
    value: function getEndToEndDeviceData(txn, func) {
      func(getJsonItem(this.store, KEY_DEVICE_DATA));
    }
  }, {
    key: "storeEndToEndDeviceData",
    value: function storeEndToEndDeviceData(deviceData, txn) {
      setJsonItem(this.store, KEY_DEVICE_DATA, deviceData);
    }
  }, {
    key: "storeEndToEndRoom",
    value: function storeEndToEndRoom(roomId, roomInfo, txn) {
      setJsonItem(this.store, keyEndToEndRoomsPrefix(roomId), roomInfo);
    }
  }, {
    key: "getEndToEndRooms",
    value: function getEndToEndRooms(txn, func) {
      var result = {};
      var prefix = keyEndToEndRoomsPrefix('');

      for (var i = 0; i < this.store.length; ++i) {
        var key = this.store.key(i);

        if (key.startsWith(prefix)) {
          var roomId = key.substr(prefix.length);
          result[roomId] = getJsonItem(this.store, key);
        }
      }

      func(result);
    }
  }, {
    key: "getSessionsNeedingBackup",
    value: function getSessionsNeedingBackup(limit) {
      var _this2 = this;

      var sessionsNeedingBackup = getJsonItem(this.store, KEY_SESSIONS_NEEDING_BACKUP) || {};
      var sessions = [];

      for (var session in sessionsNeedingBackup) {
        if (Object.prototype.hasOwnProperty.call(sessionsNeedingBackup, session)) {
          var _ret = function () {
            // see getAllEndToEndInboundGroupSessions for the magic number explanations
            var senderKey = session.substr(0, 43);
            var sessionId = session.substr(44);

            _this2.getEndToEndInboundGroupSession(senderKey, sessionId, null, function (sessionData) {
              sessions.push({
                senderKey: senderKey,
                sessionId: sessionId,
                sessionData: sessionData
              });
            });

            if (limit && session.length >= limit) {
              return "break";
            }
          }();

          if (_ret === "break") break;
        }
      }

      return Promise.resolve(sessions);
    }
  }, {
    key: "countSessionsNeedingBackup",
    value: function countSessionsNeedingBackup() {
      var sessionsNeedingBackup = getJsonItem(this.store, KEY_SESSIONS_NEEDING_BACKUP) || {};
      return Promise.resolve(Object.keys(sessionsNeedingBackup).length);
    }
  }, {
    key: "unmarkSessionsNeedingBackup",
    value: function unmarkSessionsNeedingBackup(sessions) {
      var sessionsNeedingBackup = getJsonItem(this.store, KEY_SESSIONS_NEEDING_BACKUP) || {};
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = sessions[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var session = _step3.value;
          delete sessionsNeedingBackup[session.senderKey + '/' + session.sessionId];
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3["return"] != null) {
            _iterator3["return"]();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      setJsonItem(this.store, KEY_SESSIONS_NEEDING_BACKUP, sessionsNeedingBackup);
      return Promise.resolve();
    }
  }, {
    key: "markSessionsNeedingBackup",
    value: function markSessionsNeedingBackup(sessions) {
      var sessionsNeedingBackup = getJsonItem(this.store, KEY_SESSIONS_NEEDING_BACKUP) || {};
      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = sessions[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var session = _step4.value;
          sessionsNeedingBackup[session.senderKey + '/' + session.sessionId] = true;
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4["return"] != null) {
            _iterator4["return"]();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }

      setJsonItem(this.store, KEY_SESSIONS_NEEDING_BACKUP, sessionsNeedingBackup);
      return Promise.resolve();
    }
    /**
     * Delete all data from this store.
     *
     * @returns {Promise} Promise which resolves when the store has been cleared.
     */

  }, {
    key: "deleteAllData",
    value: function deleteAllData() {
      this.store.removeItem(KEY_END_TO_END_ACCOUNT);
      return Promise.resolve();
    } // Olm account

  }, {
    key: "getAccount",
    value: function getAccount(txn, func) {
      var account = getJsonItem(this.store, KEY_END_TO_END_ACCOUNT);
      func(account);
    }
  }, {
    key: "storeAccount",
    value: function storeAccount(txn, newData) {
      setJsonItem(this.store, KEY_END_TO_END_ACCOUNT, newData);
    }
  }, {
    key: "getCrossSigningKeys",
    value: function getCrossSigningKeys(txn, func) {
      var keys = getJsonItem(this.store, KEY_CROSS_SIGNING_KEYS);
      func(keys);
    }
  }, {
    key: "getSecretStorePrivateKey",
    value: function getSecretStorePrivateKey(txn, func, type) {
      var key = getJsonItem(this.store, E2E_PREFIX + "ssss_cache.".concat(type));
      func(key ? Uint8Array.from(key) : key);
    }
  }, {
    key: "storeCrossSigningKeys",
    value: function storeCrossSigningKeys(txn, keys) {
      setJsonItem(this.store, KEY_CROSS_SIGNING_KEYS, keys);
    }
  }, {
    key: "storeSecretStorePrivateKey",
    value: function storeSecretStorePrivateKey(txn, type, key) {
      setJsonItem(this.store, E2E_PREFIX + "ssss_cache.".concat(type), Array.from(key));
    }
  }, {
    key: "doTxn",
    value: function doTxn(mode, stores, func) {
      return Promise.resolve(func(null));
    }
  }], [{
    key: "exists",
    value: function exists(webStore) {
      var length = webStore.length;

      for (var i = 0; i < length; i++) {
        if (webStore.key(i).startsWith(E2E_PREFIX)) {
          return true;
        }
      }

      return false;
    }
  }]);
  return LocalStorageCryptoStore;
}(_memoryCryptoStore.MemoryCryptoStore);

exports.LocalStorageCryptoStore = LocalStorageCryptoStore;

function getJsonItem(store, key) {
  try {
    // if the key is absent, store.getItem() returns null, and
    // JSON.parse(null) === null, so this returns null.
    return JSON.parse(store.getItem(key));
  } catch (e) {
    _logger.logger.log("Error: Failed to get key %s: %s", key, e.stack || e);

    _logger.logger.log(e.stack);
  }

  return null;
}

function setJsonItem(store, key, val) {
  store.setItem(key, JSON.stringify(val));
}