"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MemoryCryptoStore = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _logger = require("../../logger");

var utils = _interopRequireWildcard(require("../../utils"));

/*
Copyright 2017 Vector Creations Ltd
Copyright 2018 New Vector Ltd
Copyright 2020 The Matrix.org Foundation C.I.C.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Internal module. in-memory storage for e2e.
 *
 * @module
 */

/**
 * @implements {module:crypto/store/base~CryptoStore}
 */
var MemoryCryptoStore =
/*#__PURE__*/
function () {
  function MemoryCryptoStore() {
    (0, _classCallCheck2["default"])(this, MemoryCryptoStore);
    this._outgoingRoomKeyRequests = [];
    this._account = null;
    this._crossSigningKeys = null;
    this._privateKeys = {};
    this._backupKeys = {}; // Map of {devicekey -> {sessionId -> session pickle}}

    this._sessions = {}; // Map of {devicekey -> array of problems}

    this._sessionProblems = {}; // Map of {userId -> deviceId -> true}

    this._notifiedErrorDevices = {}; // Map of {senderCurve25519Key+'/'+sessionId -> session data object}

    this._inboundGroupSessions = {};
    this._inboundGroupSessionsWithheld = {}; // Opaque device data object

    this._deviceData = null; // roomId -> Opaque roomInfo object

    this._rooms = {}; // Set of {senderCurve25519Key+'/'+sessionId}

    this._sessionsNeedingBackup = {};
  }
  /**
   * Ensure the database exists and is up-to-date.
   *
   * This must be called before the store can be used.
   *
   * @return {Promise} resolves to the store.
   */


  (0, _createClass2["default"])(MemoryCryptoStore, [{
    key: "startup",
    value: function () {
      var _startup = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee() {
        return _regenerator["default"].wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                return _context.abrupt("return", this);

              case 1:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function startup() {
        return _startup.apply(this, arguments);
      }

      return startup;
    }()
    /**
     * Delete all data from this store.
     *
     * @returns {Promise} Promise which resolves when the store has been cleared.
     */

  }, {
    key: "deleteAllData",
    value: function deleteAllData() {
      return Promise.resolve();
    }
    /**
     * Look for an existing outgoing room key request, and if none is found,
     * add a new one
     *
     * @param {module:crypto/store/base~OutgoingRoomKeyRequest} request
     *
     * @returns {Promise} resolves to
     *    {@link module:crypto/store/base~OutgoingRoomKeyRequest}: either the
     *    same instance as passed in, or the existing one.
     */

  }, {
    key: "getOrAddOutgoingRoomKeyRequest",
    value: function getOrAddOutgoingRoomKeyRequest(request) {
      var _this = this;

      var requestBody = request.requestBody;
      return utils.promiseTry(function () {
        // first see if we already have an entry for this request.
        var existing = _this._getOutgoingRoomKeyRequest(requestBody);

        if (existing) {
          // this entry matches the request - return it.
          _logger.logger.log("already have key request outstanding for " + "".concat(requestBody.room_id, " / ").concat(requestBody.session_id, ": ") + "not sending another");

          return existing;
        } // we got to the end of the list without finding a match
        // - add the new request.


        _logger.logger.log("enqueueing key request for ".concat(requestBody.room_id, " / ") + requestBody.session_id);

        _this._outgoingRoomKeyRequests.push(request);

        return request;
      });
    }
    /**
     * Look for an existing room key request
     *
     * @param {module:crypto~RoomKeyRequestBody} requestBody
     *    existing request to look for
     *
     * @return {Promise} resolves to the matching
     *    {@link module:crypto/store/base~OutgoingRoomKeyRequest}, or null if
     *    not found
     */

  }, {
    key: "getOutgoingRoomKeyRequest",
    value: function getOutgoingRoomKeyRequest(requestBody) {
      return Promise.resolve(this._getOutgoingRoomKeyRequest(requestBody));
    }
    /**
     * Looks for existing room key request, and returns the result synchronously.
     *
     * @internal
     *
     * @param {module:crypto~RoomKeyRequestBody} requestBody
     *    existing request to look for
     *
     * @return {module:crypto/store/base~OutgoingRoomKeyRequest?}
     *    the matching request, or null if not found
     */

  }, {
    key: "_getOutgoingRoomKeyRequest",
    value: function _getOutgoingRoomKeyRequest(requestBody) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = this._outgoingRoomKeyRequests[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var existing = _step.value;

          if (utils.deepCompare(existing.requestBody, requestBody)) {
            return existing;
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator["return"] != null) {
            _iterator["return"]();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return null;
    }
    /**
     * Look for room key requests by state
     *
     * @param {Array<Number>} wantedStates list of acceptable states
     *
     * @return {Promise} resolves to the a
     *    {@link module:crypto/store/base~OutgoingRoomKeyRequest}, or null if
     *    there are no pending requests in those states
     */

  }, {
    key: "getOutgoingRoomKeyRequestByState",
    value: function getOutgoingRoomKeyRequestByState(wantedStates) {
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = this._outgoingRoomKeyRequests[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var req = _step2.value;
          var _iteratorNormalCompletion3 = true;
          var _didIteratorError3 = false;
          var _iteratorError3 = undefined;

          try {
            for (var _iterator3 = wantedStates[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
              var state = _step3.value;

              if (req.state === state) {
                return Promise.resolve(req);
              }
            }
          } catch (err) {
            _didIteratorError3 = true;
            _iteratorError3 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion3 && _iterator3["return"] != null) {
                _iterator3["return"]();
              }
            } finally {
              if (_didIteratorError3) {
                throw _iteratorError3;
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
            _iterator2["return"]();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      return Promise.resolve(null);
    }
    /**
     *
     * @param {Number} wantedState
     * @return {Promise<Array<*>>} All OutgoingRoomKeyRequests in state
     */

  }, {
    key: "getAllOutgoingRoomKeyRequestsByState",
    value: function getAllOutgoingRoomKeyRequestsByState(wantedState) {
      return Promise.resolve(this._outgoingRoomKeyRequests.filter(function (r) {
        return r.state == wantedState;
      }));
    }
  }, {
    key: "getOutgoingRoomKeyRequestsByTarget",
    value: function getOutgoingRoomKeyRequestsByTarget(userId, deviceId, wantedStates) {
      var results = [];
      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = this._outgoingRoomKeyRequests[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var req = _step4.value;
          var _iteratorNormalCompletion5 = true;
          var _didIteratorError5 = false;
          var _iteratorError5 = undefined;

          try {
            for (var _iterator5 = wantedStates[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
              var state = _step5.value;

              if (req.state === state && req.recipients.includes({
                userId: userId,
                deviceId: deviceId
              })) {
                results.push(req);
              }
            }
          } catch (err) {
            _didIteratorError5 = true;
            _iteratorError5 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion5 && _iterator5["return"] != null) {
                _iterator5["return"]();
              }
            } finally {
              if (_didIteratorError5) {
                throw _iteratorError5;
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4["return"] != null) {
            _iterator4["return"]();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }

      return Promise.resolve(results);
    }
    /**
     * Look for an existing room key request by id and state, and update it if
     * found
     *
     * @param {string} requestId      ID of request to update
     * @param {number} expectedState  state we expect to find the request in
     * @param {Object} updates        name/value map of updates to apply
     *
     * @returns {Promise} resolves to
     *    {@link module:crypto/store/base~OutgoingRoomKeyRequest}
     *    updated request, or null if no matching row was found
     */

  }, {
    key: "updateOutgoingRoomKeyRequest",
    value: function updateOutgoingRoomKeyRequest(requestId, expectedState, updates) {
      var _iteratorNormalCompletion6 = true;
      var _didIteratorError6 = false;
      var _iteratorError6 = undefined;

      try {
        for (var _iterator6 = this._outgoingRoomKeyRequests[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
          var req = _step6.value;

          if (req.requestId !== requestId) {
            continue;
          }

          if (req.state != expectedState) {
            _logger.logger.warn("Cannot update room key request from ".concat(expectedState, " ") + "as it was already updated to ".concat(req.state));

            return Promise.resolve(null);
          }

          Object.assign(req, updates);
          return Promise.resolve(req);
        }
      } catch (err) {
        _didIteratorError6 = true;
        _iteratorError6 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion6 && _iterator6["return"] != null) {
            _iterator6["return"]();
          }
        } finally {
          if (_didIteratorError6) {
            throw _iteratorError6;
          }
        }
      }

      return Promise.resolve(null);
    }
    /**
     * Look for an existing room key request by id and state, and delete it if
     * found
     *
     * @param {string} requestId      ID of request to update
     * @param {number} expectedState  state we expect to find the request in
     *
     * @returns {Promise} resolves once the operation is completed
     */

  }, {
    key: "deleteOutgoingRoomKeyRequest",
    value: function deleteOutgoingRoomKeyRequest(requestId, expectedState) {
      for (var i = 0; i < this._outgoingRoomKeyRequests.length; i++) {
        var req = this._outgoingRoomKeyRequests[i];

        if (req.requestId !== requestId) {
          continue;
        }

        if (req.state != expectedState) {
          _logger.logger.warn("Cannot delete room key request in state ".concat(req.state, " ") + "(expected ".concat(expectedState, ")"));

          return Promise.resolve(null);
        }

        this._outgoingRoomKeyRequests.splice(i, 1);

        return Promise.resolve(req);
      }

      return Promise.resolve(null);
    } // Olm Account

  }, {
    key: "getAccount",
    value: function getAccount(txn, func) {
      func(this._account);
    }
  }, {
    key: "storeAccount",
    value: function storeAccount(txn, newData) {
      this._account = newData;
    }
  }, {
    key: "getCrossSigningKeys",
    value: function getCrossSigningKeys(txn, func) {
      func(this._crossSigningKeys);
    }
  }, {
    key: "getSecretStorePrivateKey",
    value: function getSecretStorePrivateKey(txn, func, type) {
      var result = this._privateKeys[type];
      return func(result || null);
    }
  }, {
    key: "storeCrossSigningKeys",
    value: function storeCrossSigningKeys(txn, keys) {
      this._crossSigningKeys = keys;
    }
  }, {
    key: "storeSecretStorePrivateKey",
    value: function storeSecretStorePrivateKey(txn, type, key) {
      this._privateKeys[type] = key;
    } // Olm Sessions

  }, {
    key: "countEndToEndSessions",
    value: function countEndToEndSessions(txn, func) {
      return Object.keys(this._sessions).length;
    }
  }, {
    key: "getEndToEndSession",
    value: function getEndToEndSession(deviceKey, sessionId, txn, func) {
      var deviceSessions = this._sessions[deviceKey] || {};
      func(deviceSessions[sessionId] || null);
    }
  }, {
    key: "getEndToEndSessions",
    value: function getEndToEndSessions(deviceKey, txn, func) {
      func(this._sessions[deviceKey] || {});
    }
  }, {
    key: "getAllEndToEndSessions",
    value: function getAllEndToEndSessions(txn, func) {
      Object.entries(this._sessions).forEach(function (_ref) {
        var _ref2 = (0, _slicedToArray2["default"])(_ref, 2),
            deviceKey = _ref2[0],
            deviceSessions = _ref2[1];

        Object.entries(deviceSessions).forEach(function (_ref3) {
          var _ref4 = (0, _slicedToArray2["default"])(_ref3, 2),
              sessionId = _ref4[0],
              session = _ref4[1];

          func((0, _objectSpread2["default"])({}, session, {
            deviceKey: deviceKey,
            sessionId: sessionId
          }));
        });
      });
    }
  }, {
    key: "storeEndToEndSession",
    value: function storeEndToEndSession(deviceKey, sessionId, sessionInfo, txn) {
      var deviceSessions = this._sessions[deviceKey];

      if (deviceSessions === undefined) {
        deviceSessions = {};
        this._sessions[deviceKey] = deviceSessions;
      }

      deviceSessions[sessionId] = sessionInfo;
    }
  }, {
    key: "storeEndToEndSessionProblem",
    value: function () {
      var _storeEndToEndSessionProblem = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee2(deviceKey, type, fixed) {
        var problems;
        return _regenerator["default"].wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                problems = this._sessionProblems[deviceKey] = this._sessionProblems[deviceKey] || [];
                problems.push({
                  type: type,
                  fixed: fixed,
                  time: Date.now()
                });
                problems.sort(function (a, b) {
                  return a.time - b.time;
                });

              case 3:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function storeEndToEndSessionProblem(_x, _x2, _x3) {
        return _storeEndToEndSessionProblem.apply(this, arguments);
      }

      return storeEndToEndSessionProblem;
    }()
  }, {
    key: "getEndToEndSessionProblem",
    value: function () {
      var _getEndToEndSessionProblem = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee3(deviceKey, timestamp) {
        var problems, lastProblem, _iteratorNormalCompletion7, _didIteratorError7, _iteratorError7, _iterator7, _step7, problem;

        return _regenerator["default"].wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                problems = this._sessionProblems[deviceKey] || [];

                if (problems.length) {
                  _context3.next = 3;
                  break;
                }

                return _context3.abrupt("return", null);

              case 3:
                lastProblem = problems[problems.length - 1];
                _iteratorNormalCompletion7 = true;
                _didIteratorError7 = false;
                _iteratorError7 = undefined;
                _context3.prev = 7;
                _iterator7 = problems[Symbol.iterator]();

              case 9:
                if (_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done) {
                  _context3.next = 16;
                  break;
                }

                problem = _step7.value;

                if (!(problem.time > timestamp)) {
                  _context3.next = 13;
                  break;
                }

                return _context3.abrupt("return", Object.assign({}, problem, {
                  fixed: lastProblem.fixed
                }));

              case 13:
                _iteratorNormalCompletion7 = true;
                _context3.next = 9;
                break;

              case 16:
                _context3.next = 22;
                break;

              case 18:
                _context3.prev = 18;
                _context3.t0 = _context3["catch"](7);
                _didIteratorError7 = true;
                _iteratorError7 = _context3.t0;

              case 22:
                _context3.prev = 22;
                _context3.prev = 23;

                if (!_iteratorNormalCompletion7 && _iterator7["return"] != null) {
                  _iterator7["return"]();
                }

              case 25:
                _context3.prev = 25;

                if (!_didIteratorError7) {
                  _context3.next = 28;
                  break;
                }

                throw _iteratorError7;

              case 28:
                return _context3.finish(25);

              case 29:
                return _context3.finish(22);

              case 30:
                if (!lastProblem.fixed) {
                  _context3.next = 34;
                  break;
                }

                return _context3.abrupt("return", null);

              case 34:
                return _context3.abrupt("return", lastProblem);

              case 35:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this, [[7, 18, 22, 30], [23,, 25, 29]]);
      }));

      function getEndToEndSessionProblem(_x4, _x5) {
        return _getEndToEndSessionProblem.apply(this, arguments);
      }

      return getEndToEndSessionProblem;
    }()
  }, {
    key: "filterOutNotifiedErrorDevices",
    value: function () {
      var _filterOutNotifiedErrorDevices = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee4(devices) {
        var notifiedErrorDevices, ret, _iteratorNormalCompletion8, _didIteratorError8, _iteratorError8, _iterator8, _step8, device, userId, deviceInfo;

        return _regenerator["default"].wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                notifiedErrorDevices = this._notifiedErrorDevices;
                ret = [];
                _iteratorNormalCompletion8 = true;
                _didIteratorError8 = false;
                _iteratorError8 = undefined;
                _context4.prev = 5;

                for (_iterator8 = devices[Symbol.iterator](); !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
                  device = _step8.value;
                  userId = device.userId, deviceInfo = device.deviceInfo;

                  if (userId in notifiedErrorDevices) {
                    if (!(deviceInfo.deviceId in notifiedErrorDevices[userId])) {
                      ret.push(device);
                      notifiedErrorDevices[userId][deviceInfo.deviceId] = true;
                    }
                  } else {
                    ret.push(device);
                    notifiedErrorDevices[userId] = (0, _defineProperty2["default"])({}, deviceInfo.deviceId, true);
                  }
                }

                _context4.next = 13;
                break;

              case 9:
                _context4.prev = 9;
                _context4.t0 = _context4["catch"](5);
                _didIteratorError8 = true;
                _iteratorError8 = _context4.t0;

              case 13:
                _context4.prev = 13;
                _context4.prev = 14;

                if (!_iteratorNormalCompletion8 && _iterator8["return"] != null) {
                  _iterator8["return"]();
                }

              case 16:
                _context4.prev = 16;

                if (!_didIteratorError8) {
                  _context4.next = 19;
                  break;
                }

                throw _iteratorError8;

              case 19:
                return _context4.finish(16);

              case 20:
                return _context4.finish(13);

              case 21:
                return _context4.abrupt("return", ret);

              case 22:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this, [[5, 9, 13, 21], [14,, 16, 20]]);
      }));

      function filterOutNotifiedErrorDevices(_x6) {
        return _filterOutNotifiedErrorDevices.apply(this, arguments);
      }

      return filterOutNotifiedErrorDevices;
    }() // Inbound Group Sessions

  }, {
    key: "getEndToEndInboundGroupSession",
    value: function getEndToEndInboundGroupSession(senderCurve25519Key, sessionId, txn, func) {
      var k = senderCurve25519Key + '/' + sessionId;
      func(this._inboundGroupSessions[k] || null, this._inboundGroupSessionsWithheld[k] || null);
    }
  }, {
    key: "getAllEndToEndInboundGroupSessions",
    value: function getAllEndToEndInboundGroupSessions(txn, func) {
      for (var _i = 0, _Object$keys = Object.keys(this._inboundGroupSessions); _i < _Object$keys.length; _i++) {
        var key = _Object$keys[_i];
        // we can't use split, as the components we are trying to split out
        // might themselves contain '/' characters. We rely on the
        // senderKey being a (32-byte) curve25519 key, base64-encoded
        // (hence 43 characters long).
        func({
          senderKey: key.substr(0, 43),
          sessionId: key.substr(44),
          sessionData: this._inboundGroupSessions[key]
        });
      }

      func(null);
    }
  }, {
    key: "addEndToEndInboundGroupSession",
    value: function addEndToEndInboundGroupSession(senderCurve25519Key, sessionId, sessionData, txn) {
      var k = senderCurve25519Key + '/' + sessionId;

      if (this._inboundGroupSessions[k] === undefined) {
        this._inboundGroupSessions[k] = sessionData;
      }
    }
  }, {
    key: "storeEndToEndInboundGroupSession",
    value: function storeEndToEndInboundGroupSession(senderCurve25519Key, sessionId, sessionData, txn) {
      this._inboundGroupSessions[senderCurve25519Key + '/' + sessionId] = sessionData;
    }
  }, {
    key: "storeEndToEndInboundGroupSessionWithheld",
    value: function storeEndToEndInboundGroupSessionWithheld(senderCurve25519Key, sessionId, sessionData, txn) {
      var k = senderCurve25519Key + '/' + sessionId;
      this._inboundGroupSessionsWithheld[k] = sessionData;
    } // Device Data

  }, {
    key: "getEndToEndDeviceData",
    value: function getEndToEndDeviceData(txn, func) {
      func(this._deviceData);
    }
  }, {
    key: "storeEndToEndDeviceData",
    value: function storeEndToEndDeviceData(deviceData, txn) {
      this._deviceData = deviceData;
    } // E2E rooms

  }, {
    key: "storeEndToEndRoom",
    value: function storeEndToEndRoom(roomId, roomInfo, txn) {
      this._rooms[roomId] = roomInfo;
    }
  }, {
    key: "getEndToEndRooms",
    value: function getEndToEndRooms(txn, func) {
      func(this._rooms);
    }
  }, {
    key: "getSessionsNeedingBackup",
    value: function getSessionsNeedingBackup(limit) {
      var sessions = [];

      for (var session in this._sessionsNeedingBackup) {
        if (this._inboundGroupSessions[session]) {
          sessions.push({
            senderKey: session.substr(0, 43),
            sessionId: session.substr(44),
            sessionData: this._inboundGroupSessions[session]
          });

          if (limit && session.length >= limit) {
            break;
          }
        }
      }

      return Promise.resolve(sessions);
    }
  }, {
    key: "countSessionsNeedingBackup",
    value: function countSessionsNeedingBackup() {
      return Promise.resolve(Object.keys(this._sessionsNeedingBackup).length);
    }
  }, {
    key: "unmarkSessionsNeedingBackup",
    value: function unmarkSessionsNeedingBackup(sessions) {
      var _iteratorNormalCompletion9 = true;
      var _didIteratorError9 = false;
      var _iteratorError9 = undefined;

      try {
        for (var _iterator9 = sessions[Symbol.iterator](), _step9; !(_iteratorNormalCompletion9 = (_step9 = _iterator9.next()).done); _iteratorNormalCompletion9 = true) {
          var session = _step9.value;
          var sessionKey = session.senderKey + '/' + session.sessionId;
          delete this._sessionsNeedingBackup[sessionKey];
        }
      } catch (err) {
        _didIteratorError9 = true;
        _iteratorError9 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion9 && _iterator9["return"] != null) {
            _iterator9["return"]();
          }
        } finally {
          if (_didIteratorError9) {
            throw _iteratorError9;
          }
        }
      }

      return Promise.resolve();
    }
  }, {
    key: "markSessionsNeedingBackup",
    value: function markSessionsNeedingBackup(sessions) {
      var _iteratorNormalCompletion10 = true;
      var _didIteratorError10 = false;
      var _iteratorError10 = undefined;

      try {
        for (var _iterator10 = sessions[Symbol.iterator](), _step10; !(_iteratorNormalCompletion10 = (_step10 = _iterator10.next()).done); _iteratorNormalCompletion10 = true) {
          var session = _step10.value;
          var sessionKey = session.senderKey + '/' + session.sessionId;
          this._sessionsNeedingBackup[sessionKey] = true;
        }
      } catch (err) {
        _didIteratorError10 = true;
        _iteratorError10 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion10 && _iterator10["return"] != null) {
            _iterator10["return"]();
          }
        } finally {
          if (_didIteratorError10) {
            throw _iteratorError10;
          }
        }
      }

      return Promise.resolve();
    } // Session key backups

  }, {
    key: "doTxn",
    value: function doTxn(mode, stores, func) {
      return Promise.resolve(func(null));
    }
  }]);
  return MemoryCryptoStore;
}();

exports.MemoryCryptoStore = MemoryCryptoStore;